# Copyright (C) 2023-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This file is part of the GDB testsuite.
# It tests Python-based pretty-printing of stubs.

import gdb.printing


class SPrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        char_ptr = gdb.lookup_type("char").pointer()
        int_ptr = gdb.lookup_type("int").pointer()
        # m_i should be after the vtable, which has the size of a pointer
        i = (
            (self.val.address.cast(char_ptr) + int_ptr.sizeof)
            .cast(int_ptr)
            .dereference()
        )
        return "{pp m_i = %d}" % int(i)


pp = gdb.printing.RegexpCollectionPrettyPrinter("S")
pp.add_printer("S", "^S$", SPrinter)
gdb.printing.register_pretty_printer(gdb.current_objfile(), pp, True)



# Copyright (C) 2024-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
from enum import Enum

import gdb
from gdb.missing_objfile import MissingObjfileHandler

# A global log that is filled in by instances of the LOG_HANDLER class
# when they are called.
handler_call_log = []

# A global holding a string, the build-id of the last missing objfile
# which triggered the 'handler' class below.  This is set in the
# __call__ method of the 'handler' class and then checked from the
# expect script.
handler_last_buildid = None


# A global holding a string, the filename of the last missing objfile
# which triggered the 'handler' class below.  This is set in the
# __call__ method of the 'handler' class and then checked from the
# expect script.
handler_last_filename = None


# A helper function that makes some assertions about the arguments
# passed to a MissingObjfileHandler.__call__() method.
def check_args(pspace, buildid, filename):
    assert type(filename) == str
    assert filename != ""
    assert type(pspace) == gdb.Progspace
    assert type(buildid) == str
    assert buildid != ""


# Enum used to configure the 'handler' class from the test script.
class Mode(Enum):
    RETURN_NONE = 0
    RETURN_TRUE = 1
    RETURN_FALSE = 2
    RETURN_STRING = 3


# A missing objfile handler which can be configured to return each of
# the different possible return types.
class handler(MissingObjfileHandler):
    def __init__(self):
        super().__init__("handler")
        self._call_count = 0
        self._mode = Mode.RETURN_NONE

    def __call__(self, pspace, buildid, filename):
        global handler_call_log, handler_last_buildid, handler_last_filename
        check_args(pspace, buildid, filename)
        handler_call_log.append(self.name)
        handler_last_buildid = buildid
        handler_last_filename = filename
        self._call_count += 1
        if self._mode == Mode.RETURN_NONE:
            return None

        if self._mode == Mode.RETURN_TRUE:
            shutil.copy(self._src, self._dest)

            # If we're using the fission-dwp board then there will
            # also be a .dwp file that needs to be copied.
            dwp_src = self._src + ".dwp"
            if os.path.exists(dwp_src):
                dwp_dest = self._dest + ".dwp"
                shutil.copy(dwp_src, dwp_dest)

            return True

        if self._mode == Mode.RETURN_FALSE:
            return False

        if self._mode == Mode.RETURN_STRING:
            return self._dest

        assert False

    @property
    def call_count(self):
        """Return a count, the number of calls to __call__ since the last
        call to set_mode.
        """
        return self._call_count

    def set_mode(self, mode, *args):
        self._call_count = 0
        self._mode = mode

        if mode == Mode.RETURN_NONE:
            assert len(args) == 0
            return

        if mode == Mode.RETURN_TRUE:
            assert len(args) == 2
            self._src = args[0]
            self._dest = args[1]
            return

        if mode == Mode.RETURN_FALSE:
            assert len(args) == 0
            return

        if mode == Mode.RETURN_STRING:
            assert len(args) == 1
            self._dest = args[0]
            return

        assert False


# A missing objfile handler which raises an exception.  The type of
# exception to be raised is configured from the test script.
class exception_handler(MissingObjfileHandler):
    def __init__(self):
        super().__init__("exception_handler")
        self.exception_type = None

    def __call__(self, pspace, buildid, filename):
        global handler_call_log
        check_args(pspace, buildid, filename)
        handler_call_log.append(self.name)
        assert self.exception_type is not None
        raise self.exception_type("message")


# A very simple logging missing objfile handler.  Always returns None
# so that GDB will try any other registered handlers, but first logs
# the name of this handler into the global HANDLER_CALL_LOG, which can
# then be checked from the test script.
class log_handler(MissingObjfileHandler):
    def __call__(self, pspace, buildid, filename):
        global handler_call_log
        check_args(pspace, buildid, filename)
        handler_call_log.append(self.name)
        return None


# A basic helper function, this keeps lines shorter in the TCL script.
def register(name, locus=None):
    gdb.missing_objfile.register_handler(locus, log_handler(name))


# Create instances of the handlers, but don't install any.  We install
# these as needed from the TCL script.
rhandler = exception_handler()
handler_obj = handler()

print("Success")



# Copyright (C) 2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This file is part of the GDB testsuite.

import gdb


class PyCommandsBreakpoint(gdb.Breakpoint):
    bp_dict = dict()

    def __init__(self, *args, **kwargs):
        gdb.Breakpoint.__init__(self, *args, **kwargs)
        PyCommandsBreakpoint.bp_dict[self.number] = self
        self.commands = ""

    @staticmethod
    def run_py_commands(num):
        bp = PyCommandsBreakpoint.bp_dict[num]
        if hasattr(bp, "py_commands"):
            bp.py_commands()

    def __setattr__(self, name, value):
        if name == "commands":
            l = ["python PyCommandsBreakpoint.run_py_commands(%d)" % self.number]
            if value != "":
                l.append(value)
            value = "\n".join(l)

        super().__setattr__(name, value)


class TestBreakpoint(PyCommandsBreakpoint):
    def __init__(self):
        PyCommandsBreakpoint.__init__(self, spec="test")
        self.var = 1
        self.commands = "continue"
        self.silent = True

    def stop(self):
        if self.var == 3:
            self.commands = ""
        return True

    def py_commands(self):
        print("VAR: %d" % self.var)
        self.var += 1



# Copyright 2023-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Test the type checker.

import typing

from gdb.dap.typecheck import type_check


# A wrapper to call a function that should cause a type-checking
# failure.  Returns if the exception was seen.  Throws an exception if
# a TypeError was not seen.
def should_fail(func, **args):
    try:
        func(**args)
    except TypeError:
        return
    raise RuntimeError("function failed to throw TypeError")


# Also specify a return type to make sure return types do not confuse
# the checker.
@type_check
def simple_types(*, b: bool, s: str, i: int = 23) -> int:
    return i


def check_simple():
    simple_types(b=True, s="DEI", i=97)
    # Check the absence of a defaulted argument.
    simple_types(b=True, s="DEI")
    simple_types(b=False, s="DEI", i=97)
    should_fail(simple_types, b=97, s="DEI", i=97)
    should_fail(simple_types, b=True, s=None, i=97)
    should_fail(simple_types, b=True, s="DEI", i={})


@type_check
def sequence_type(*, s: typing.Sequence[str]):
    pass


def check_sequence():
    sequence_type(s=("hi", "out", "there"))
    sequence_type(s=["hi", "out", "there"])
    sequence_type(s=())
    sequence_type(s=[])
    should_fail(sequence_type, s=23)
    should_fail(sequence_type, s=["hi", 97])


@type_check
def map_type(*, m: typing.Mapping[str, int]):
    pass


def check_map():
    map_type(m={})
    map_type(m={"dei": 23})
    should_fail(map_type, m=[1, 2, 3])
    should_fail(map_type, m=None)
    should_fail(map_type, m={"dei": "string"})


@type_check
def opt_type(*, u: typing.Optional[int]):
    pass


def check_opt():
    opt_type(u=23)
    opt_type(u=None)
    should_fail(opt_type, u="string")


def check_everything():
    # Older versions of Python can't really implement this.
    if hasattr(typing, "get_origin"):
        # Just let any exception propagate and end up in the log.
        check_simple()
        check_sequence()
        check_map()
        check_opt()
        print("OK")
    else:
        print("UNSUPPORTED")



# Copyright (C) 2023-2025 Free Software Foundation, Inc.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class TestWindow:
    def __init__(self, tui_win, msg):
        self.msg = msg
        self.tui_win = tui_win
        print("Entering TestWindow.__init__: %s" % self.msg)

    def render(self):
        self.tui_win.erase()
        self.tui_win.write("TestWindow (%s)" % self.msg)

    def __del__(self):
        print("Entering TestWindow.__del__: %s" % self.msg)


class TestWindowFactory:
    def __init__(self, msg):
        self.msg = msg
        print("Entering TestWindowFactory.__init__: %s" % self.msg)

    def __call__(self, tui_win):
        print("Entering TestWindowFactory.__call__: %s" % self.msg)
        return TestWindow(tui_win, self.msg)

    def __del__(self):
        print("Entering TestWindowFactory.__del__: %s" % self.msg)


def register_window_factory(msg):
    gdb.register_window_type("test_window", TestWindowFactory(msg))


print("Python script imported")
